const Conf = require('conf');
const paths = require('@yif/cli-paths');
const glob = require('glob');
const path = require('path');
const columnify = require('columnify');

const { console, inquirer, chalk } = yif;

class Config extends Conf {
  constructor(options) {
    super(options);
    options = {
      name: 'config',
      ...options,
    };

    options.cwd = paths.home;
    options.configName = 'config';

    this.defaultQuestions = [
      {
        type: 'input',
        name: 'dev-host',
        message: '日常环境 CDN host',
        required: true,
        default: '//assets-daily-1258060709.cos.ap-chengdu.myqcloud.com',
      },
      {
        type: 'input',
        name: 'host',
        message: '线上环境 CDN host',
        required: true,
        default: '//static.xue2shou.com',
      },
    ];
  }

  get(key) {
    const value = super.get(key);

    if (!value) {
      console.warn('不存在配置:', chalk.white(key), '请先设置');
    }

    return value;
  }

  /**
   * 获取所有已安装套件的 config 配置
   *
   * @return  {Array}  符合 inquirer 的问题对象
   */
  _getAllQuestion() {
    const defaultQuestions = this.defaultQuestions;
    const kitMains = glob.sync('./*/kit-*/index.js', { cwd: paths.kit });

    let kitConigQuestions = kitMains
      .map((kit) => path.join(paths.kit, kit))
      .filter((kit) => {
        return Boolean(require(kit).config);
      })
      .map((kit) => {
        return require(kit).config;
      });

    if (kitConigQuestions.length) {
      kitConigQuestions = kitConigQuestions.reduce((record, next) => {
        return [...record, ...next];
      }, []);
    }

    const configQuestions = [...defaultQuestions, ...kitConigQuestions].map((question) => {
      question.default = this.get(question.name) || question.default || undefined;
      return question;
    });

    return configQuestions;
  }

  async setup() {
    console.log(paths.kit);
    const configQuestions = this._getAllQuestion();

    const kv = await inquirer.prompt(configQuestions);

    this.set(kv);

    console.ok('配置设置完成');
  }

  list() {
    console.log();

    const configQuestions = this._getAllQuestion();

    const allConfigs = Object.entries(this.store).map(([key, value]) => {
      const { message = '冗余配置' } = configQuestions.find((n) => n.name == key) || {};
      return {
        field: chalk.white(key),
        value: chalk.yellow(value),
        description: chalk.gray(message),
      };
    });

    var columns = columnify(allConfigs);

    console.log(columns);
    console.log();
  }
}

module.exports = new Config();
